from panda3d.core import loadPrcFileData
#loadPrcFileData('','show-buffers 1')
loadPrcFileData('','textures-power-2 None')

from direct.showbase.ShowBase import ShowBase
from direct.task import Task
from direct.actor.Actor import Actor
from direct.interval.IntervalGlobal import *
from direct.filter.FilterManager import FilterManager
from panda3d.core import *
from math import pi, sin, cos
from vfx import vfx

MAIN_MASK = BitMask32.bit(1)
STEALTH_MASK = BitMask32.bit(17)

class MyApp(ShowBase):
    def __init__(self):
        ShowBase.__init__(self)
        #light a fire
        self.fire_node=render.attachNewNode("fireNode")    
        self.fire_node.setPos(4,0,0)
        self.fire=vfx(self.fire_node, texture='big_fire.png',scale=1.0, Z=1.2, depthTest=True, depthWrite=True)
        self.fire.loop(0.02)       
        self.fireVolume = self.loader.loadModel("fire_vol")
        self.fireVolume.reparentTo(render)
        self.fireVolume.setPos(4,0,0)
        LerpHprInterval(self.fireVolume, 2.0,(360,0,0)).loop()
        Sequence(LerpScaleInterval(self.fireVolume, 1.0,(1.2,1.2,1.2)),
                LerpScaleInterval(self.fireVolume, 1.0,(1.0,1.0,1.0))).loop()
        
        # Disable the camera trackball controls.
        self.disableMouse()
 
        # Load the environment model.
        self.environ = self.loader.loadModel("models/environment")
        # Reparent the model to render.
        self.environ.reparentTo(self.render)
        # Apply scale and position transforms on the model.
        self.environ.setScale(0.25, 0.25, 0.25)
        self.environ.setPos(-8, 42, 0)
 
        # Add the spinCameraTask procedure to the task manager.
        self.taskMgr.add(self.spinCameraTask, "SpinCameraTask")
 
        # Load and transform the panda actor.
        self.pandaActor = Actor("models/panda-model",
                                {"walk": "models/panda-walk4"})
        self.pandaActor.setScale(0.005, 0.005, 0.005)
        self.pandaActor.reparentTo(self.render)
        # Loop its animation.
        self.pandaActor.loop("walk")  
        
        # Create the four lerp intervals needed for the panda to
        # walk back and forth.
        pandaPosInterval1 = self.pandaActor.posInterval(13,
                                                        Point3(0, -10, 0),
                                                        startPos=Point3(0, 10, 0))
        pandaPosInterval2 = self.pandaActor.posInterval(13,
                                                        Point3(0, 10, 0),
                                                        startPos=Point3(0, -10, 0))
        pandaHprInterval1 = self.pandaActor.hprInterval(3,
                                                        Point3(180, 0, 0),
                                                        startHpr=Point3(0, 0, 0))
        pandaHprInterval2 = self.pandaActor.hprInterval(3,
                                                        Point3(0, 0, 0),
                                                        startHpr=Point3(180, 0, 0))
 
        # Create and play the sequence that coordinates the intervals.
        self.pandaPace = Sequence(pandaPosInterval1,
                                  pandaHprInterval1,
                                  pandaPosInterval2,
                                  pandaHprInterval2,
                                  name="pandaPace")
        self.pandaPace.loop()
        
        #new stuff HERE!
        self.distorttex=self.makeDistortTex()
        self.cam.node().setCameraMask(MAIN_MASK)
        render.setShaderInput("mask", 1.0)   
        render.setShaderInput("useMap", 0.0)   
        self.pandaActor.setShaderInput("mask", 0.0)                         
        self.pandaActor.hide(MAIN_MASK)
        self.fireVolume.hide(MAIN_MASK)
        self.fireVolume.setShaderInput("mask", 0.0)
        self.fireVolume.setShaderInput("useMap", 1.0)
        
        self.manager = FilterManager(self.win, self.cam)
        self.colortex = Texture()
        final_quad = self.manager.renderSceneInto(colortex = self.colortex)
        final_quad.setShader(Shader.load("distort.cg"))
        final_quad.setShaderInput("maincolor", self.colortex)
        final_quad.setShaderInput("distort", self.distorttex)
        final_quad.setShaderInput("amount", 0.6)
        
    # Define a procedure to move the camera.
    def spinCameraTask(self, task):
        angleDegrees = task.time * 6.0
        angleRadians = angleDegrees * (pi / 180.0)
        self.camera.setPos(20 * sin(angleRadians), -20.0 * cos(angleRadians), 3)
        self.camera.setHpr(angleDegrees, 0, 0)
        return Task.cont
        
    def makeDistortTex(self):
        normal_map = Texture()
        props = FrameBufferProperties()
        props.setRgbColor(1)
        props.setAlphaBits(1)
        mybuffer = self.win.makeTextureBuffer("Camera-norma-buff", 
                                              self.win.getXSize(), 
                                              self.win.getYSize(), 
                                              to_ram = False, 
                                              tex = normal_map, 
                                              fbp = props)
        mybuffer.setClearColor(Vec4(0,0,0,0)) 
        mybuffer.setSort(-100)
        mycamera = self.makeCamera(mybuffer)
        tempnode = NodePath(PandaNode("temp node"))
        tempnode.setShader(loader.loadShader("normals.cg"), 1)
        mycamera.node().setInitialState(tempnode.getState())
        mycamera.node().setLens(self.cam.node().getLens())
        mycamera.reparentTo(self.camera)
        return normal_map
 
app = MyApp()
app.run()